/*
Copyright (C) 1999/2000 Hendrik Lipka.
All rights reserved.

This program is free software; you can redistribute it and/or modify it under the terms 
of the GNU General Public License as published by the Free Software Foundation; 
either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY 
WARRANTY; without even the implied warranty of MERCHANTABILITY or 
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License 
for more details. 

You should have received a copy of the GNU General Public License along with this 
program; if not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
Cambridge, MA 02139, USA or see http://www.gnu.org

If you encounter problems or have a suggestion, please mail to:
hendrik.lipka@gmx.de
*/

package de.hendriklipka.ldse.client.admin;

import java.net.*;
import java.io.*;
import java.util.Vector;

public class FrameAdminHandler
{
  // Konstanten fuer die einzelnen Aktionen
  public static final int DELETEDOCUMENT=1;
  public static final int DELETESPIDER=2;
  public static final int DELETESUBSERVERBYID=3;
  public static final int DELETESUBSERVERBYNAME=4;
  public static final int GETDOCUMENTSFORSPIDER=5;
  public static final int SHUTDOWN=6;
  public static final int RECONFIG=7;
  public static final int WRITESTAT=8;
  public static final int CLEANDB=9;
  
  protected String _address=null;
  protected int _port=-1;
  protected String _password=null;
  protected String _error=frameAdminHandlerBundle.getString("OK");
  protected boolean _isError=false;
  protected String[] _documents=null;
  
  static java.util.ResourceBundle frameAdminHandlerBundle = java.util.ResourceBundle.getBundle("de.hendriklipka.ldse.client.admin.FrameAdminHandlerBundle");
  
  public FrameAdminHandler()
  {
  }
  
  public void setAddress(String s)
  {
    _address=s;
  }
  
  public void setPort(String s)
  {
    try
    {
      _port=Integer.parseInt(s);
    }
    catch(Exception ex)
    {
      _isError=true;
      _error=ex.toString();
    }
  }
  
  public void setPassword(String s)
  {
    _password=s;
  }
  
  /**
    hier wird geschaut, welche Aktion auszufuehren ist, und dann wird die passende
    Methode aufgerufen
  */
  public void execute(int action, String parameter)
  {
    switch (action)
    {
      case DELETEDOCUMENT:
           sendCommand("deld<>"+parameter,true);
           return;
      case DELETESPIDER:
           sendCommand("dels<>"+parameter,true);
           return;
      case DELETESUBSERVERBYID:
           sendCommand("delb<>"+parameter,true);
           return;
      case DELETESUBSERVERBYNAME:
           sendCommand("delb<>"+parameter,true);
           return;
      case GETDOCUMENTSFORSPIDER:
           getDocuments(parameter);
           return;
      case SHUTDOWN:
           sendCommand("shutdown",true);
           return;
      case RECONFIG:
           sendCommand("reconfig",true);
           return;
      case WRITESTAT:
           sendCommand("wrst<>"+parameter,true);
           return;
      case CLEANDB:
           sendCommand("clen",true);
           return;
    }
    _isError=false;
    _error=frameAdminHandlerBundle.getString("internal_error__unknown_action");
  }
  
  /**
    Hier wird ein uebergebener Kommando-String an den Server gesendet.
    Diese Kommandos liefern kein Ergebnis, nur "ok" oder "error".
  */
  protected void sendCommand(String command, boolean usePasswd)
  {
    Socket _socket=null;
    BufferedReader _reader=null;
    PrintWriter _writer=null;
    try
    {
      _socket=new Socket(_address,_port);
      _reader=new BufferedReader(new InputStreamReader(_socket.getInputStream()),1);
      _writer=new PrintWriter(new OutputStreamWriter(_socket.getOutputStream()));
      
      if(usePasswd)
      {
        _writer.println("admin<>"+_password);
        _writer.flush();
        String line=_reader.readLine();
        if (!line.equals("ok"))
        {
          _isError=true;
          _error=frameAdminHandlerBundle.getString("wrong_password");
          return;
        }
      }
      _writer.println(command);
      _writer.flush();
      String line=_reader.readLine();
      _writer.flush();
      if (line.equals("ok"))
      {
        _isError=false;
        return;
      }
      else
      {
        String s=frameAdminHandlerBundle.getString("error");
        line=line.substring(6);
        _isError=true;
        _error=s+": "+line;
      }
      _writer.close();
      _reader.close();
      _socket.close();
    }
    catch (Exception ex)
    {
      _isError=true;
      _error=ex.toString();
    }
  }

  /**
    Holt eine Liste von Dokumenten fuer einen Spider
  */
  protected void getDocuments(String spider)
  {
    Socket _socket=null;
    BufferedReader _reader=null;
    PrintWriter _writer=null;
    try
    {
      _socket=new Socket(_address,_port);
      _reader=new BufferedReader(new InputStreamReader(_socket.getInputStream()),1);
      _writer=new PrintWriter(new OutputStreamWriter(_socket.getOutputStream()));
      
      _writer.println("sget<>"+spider);
      _writer.flush();
      String answer=_reader.readLine();
      if (!answer.startsWith("error="))
      {
        Vector v=new Vector(10,10);
        while (true)
        {
          if (answer.equals("<><>"))
            break;
          if (null==answer)
            continue;
          v.addElement(answer);
          answer=_reader.readLine();
        }
        _documents=new String[v.size()];
        for (int i=0;i<_documents.length;i++)
        {
          _documents[i]=(String)v.elementAt(i);
        }
      }
      else
      {
        String s=frameAdminHandlerBundle.getString("error");
        answer=answer.substring(6);
        _isError=true;
        _error=s+": "+answer;
      }
      _writer.close();
      _reader.close();
      _socket.close();
    }
    catch (Exception ex)
    {
      _isError=true;
      _error=ex.toString();
    }
  }
  
  public String[] getDocuments()
  {
    return _documents;
  }
  
  public String getErrorText()
  {
    _isError=false;
    return _error;
  }
  
  public boolean isError()
  {
    return _isError;
  }
}