/*
Copyright (C) 1999/2000 Hendrik Lipka.
All rights reserved.

This program is free software; you can redistribute it and/or modify it under the terms 
of the GNU General Public License as published by the Free Software Foundation; 
either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY 
WARRANTY; without even the implied warranty of MERCHANTABILITY or 
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License 
for more details. 

You should have received a copy of the GNU General Public License along with this 
program; if not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
Cambridge, MA 02139, USA or see http://www.gnu.org

If you encounter problems or have a suggestion, please mail to:
hendrik.lipka@gmx.de
*/

package de.hendriklipka.ldse.client.ui;

import java.awt.*;
import java.util.Hashtable;
import java.io.Reader;
import java.io.Writer;

import de.hendriklipka.ldse.common.ResultListener;
import de.hendriklipka.ldse.common.ResultParser;
import de.hendriklipka.ldse.common.ResultDescriptor;
import de.hendriklipka.ldse.client.*;

/**
  Eingabefenster fuer Anfragen
*/
public class QueryFrame extends java.awt.Frame implements ResultListener
{
	public QueryFrame(FrameHandler fh, ClientConfig config)
	{
		setLayout(new BorderLayout(2,2));
		setBackground(java.awt.Color.lightGray);
		setSize(504,390);
		setVisible(false);
		panelOben.setLayout(new GridLayout(2,1,2,2));
		add("North",panelOben);
		panelOben.setBounds(0,0,504,50);
		labelSuchart.setText(queryFrameBundle.getString("labelSuchart_text"));
		panelOben.add(labelSuchart);
		labelSuchart.setBounds(0,0,168,25);
		labelMinLevel.setText(queryFrameBundle.getString("labelMinLevel_text"));
		panelOben.add(labelMinLevel);
		labelMinLevel.setBounds(168,0,168,25);
		labelMaxCount.setText(queryFrameBundle.getString("labelMaxCount_text"));
		panelOben.add(labelMaxCount);
		labelMaxCount.setBounds(336,0,168,25);
		panelOben.add(choiceSuchart);
		choiceSuchart.setBounds(0,25,168,25);
		panelOben.add(textFieldMinLevel);
		textFieldMinLevel.setBounds(168,25,168,25);
		panelOben.add(textFieldMaxCount);
		textFieldMaxCount.setBounds(336,25,168,25);
		panelPanels.setLayout(new CardLayout(0,0));
		add("Center",panelPanels);
		panelPanels.setBounds(0,50,504,294);
		panelUnten.setLayout(new GridLayout(2,1,2,2));
		add("South",panelUnten);
		panelUnten.setBounds(0,344,504,46);
		panelStatus.setLayout(new GridLayout(1,1,2,2));
		panelUnten.add(panelStatus);
		panelStatus.setBounds(0,0,504,23);
		textFieldStatus.setEditable(false);
		textFieldStatus.setCursor(java.awt.Cursor.getPredefinedCursor(java.awt.Cursor.DEFAULT_CURSOR));
		panelStatus.add(textFieldStatus);
		textFieldStatus.setForeground(java.awt.Color.black);
		textFieldStatus.setBounds(0,0,504,23);
		panelButtons.setLayout(new GridLayout(1,2,2,2));
		panelUnten.add(panelButtons);
		panelButtons.setBounds(0,23,504,23);
		buttonSuchen.setLabel(queryFrameBundle.getString("buttonSuchen_label"));
		panelButtons.add(buttonSuchen);
		buttonSuchen.setBounds(0,0,252,23);
		buttonHilfe.setLabel(queryFrameBundle.getString("buttonHilfe_label"));
		panelButtons.add(buttonHilfe);
		buttonHilfe.setBounds(252,0,252,23);
		setTitle(queryFrameBundle.getString("QueryFrame_title"));

		SymWindow aSymWindow = new SymWindow();
		this.addWindowListener(aSymWindow);
		SymAction lSymAction = new SymAction();
		buttonSuchen.addActionListener(lSymAction);
		buttonHilfe.addActionListener(lSymAction);
		SymItem lSymItem = new SymItem();
		choiceSuchart.addItemListener(lSymItem);

		try {
			choiceSuchart.select(0);
		}
		catch (IllegalArgumentException e) { }
		_parent=fh;
		_config=config;
	}

	public QueryFrame(FrameHandler fh, String title, ClientConfig config)
	{
		this(fh,config);
		setTitle(title);
	}

    public void setVisible(boolean b)
	{
		if(b)
		{
			setLocation(50, 50);
		}
		super.setVisible(b);
	}

	public void addNotify()
	{
	    Dimension d = getSize();
	    
		super.addNotify();

		if (fComponentsAdjusted)
			return;

		Insets insets = getInsets();
		setSize(insets.left + insets.right + d.width, insets.top + insets.bottom + d.height);
		Component components[] = getComponents();
		for (int i = 0; i < components.length; i++)
		{
			Point p = components[i].getLocation();
			p.translate(insets.left, insets.top);
			components[i].setLocation(p);
		}
		fComponentsAdjusted = true;
	}

	boolean fComponentsAdjusted = false;

	java.awt.Panel panelOben = new java.awt.Panel();
	java.awt.Label labelSuchart = new java.awt.Label();
	java.awt.Label labelMinLevel = new java.awt.Label();
	java.awt.Label labelMaxCount = new java.awt.Label();
	java.awt.Choice choiceSuchart = new java.awt.Choice();
	java.awt.TextField textFieldMinLevel = new java.awt.TextField();
	java.awt.TextField textFieldMaxCount = new java.awt.TextField();
	java.awt.Panel panelPanels = new java.awt.Panel();
	java.awt.Panel panelUnten = new java.awt.Panel();
	java.awt.Panel panelStatus = new java.awt.Panel();
	java.awt.TextField textFieldStatus = new java.awt.TextField();
	java.awt.Panel panelButtons = new java.awt.Panel();
	java.awt.Button buttonSuchen = new java.awt.Button();
	java.awt.Button buttonHilfe = new java.awt.Button();

	FrameHandler _parent=null;
	ClientConfig _config;
	ResultFrame _resultFrame=null;

  public void init()
  {
    clear();
  }
  
  // Eingabefelder loeschen
  public void clear()
  {
    textFieldMaxCount.setText("");
    textFieldMinLevel.setText("");
  }
  
	class SymWindow extends java.awt.event.WindowAdapter
	{
		public void windowOpened(java.awt.event.WindowEvent event)
		{
			Object object = event.getSource();
			if (object == QueryFrame.this)
				QueryFrame_WindowOpened(event);
		}

		public void windowClosing(java.awt.event.WindowEvent event)
		{
			Object object = event.getSource();
			if (object == QueryFrame.this)
				QueryFrame_WindowClosing(event);
		}
	}

	/**
	  beim Applet abmelden, damit dieses weiss, ob es ein neues Fenmster erzeugen 
	  muss oder das alte oeffnen kann
	*/
	void QueryFrame_WindowClosing(java.awt.event.WindowEvent event)
	{
	  if (null!=_parent)
	  {
  	  _parent.setQueryFrame(null);
	  }
	  this.dispose();
	  if (null==_parent)
	  {
	    System.exit(0);
	  }
	}

	class SymAction implements java.awt.event.ActionListener
	{
		public void actionPerformed(java.awt.event.ActionEvent event)
		{
			Object object = event.getSource();
			if (object == buttonSuchen)
				buttonSuchen_ActionPerformed(event);
			if (object == buttonHilfe)
				buttonHilfe_ActionPerformed(event);
		}
	}

  /**
    Durchfuehren der Suche
  */
	void buttonSuchen_ActionPerformed(java.awt.event.ActionEvent event)
	{
	  buttonSuchen.setEnabled(false);
	  try
	  {
	    // Generator holen
	    QueryGenerator qg=_oldPanel.getQueryGenerator();
	    // Limits auslesen
	    String mc=textFieldMaxCount.getText();
	    String ml=textFieldMinLevel.getText();
      int maxDocs=0;
      double minLevel=0;
      try
	    {
  	    maxDocs=Integer.parseInt(mc);
	    }
	    catch (NumberFormatException ex) {}
      try
	    {
  	    minLevel=Double.valueOf(ml).doubleValue();
	    }
	    catch (NumberFormatException ex) {}
	    // Limits setzen
	    qg.setLimits(maxDocs,minLevel);
	    try
	    {
	      // Verbindung herstellen
	      ServerConnection sc=new ServerConnection(_config);
	      sc.init();
	      Writer wr=sc.getWriter();
	      Reader r=sc.getReader();
	      // wenn Erfolg
	      if ((null!=wr)&&(null!=r))
	      {
	        // Anfrage schreiben
	        qg.writeQuery(wr);
	        // Parser erzeugen
	        ResultParser rp=new ResultParser();
	        // wenn noetig, einen neuen Frame erzeugen
	        // ansonsten wird der alte weiterverwendet
	        if (null==_resultFrame)
	        {
  	        _resultFrame=new ResultFrame(this,_config);
	        }
	        rp.init(r);
	        // Listener hinzufuegen
	        // Frame ist klar
	        rp.addResultListener(_resultFrame);
	        // auch dieses Fenster, damit wir mitzaehlen koennen (wegen der Statuszeile)
	        rp.addResultListener(this);
	        // und einlesen
	        rp.readResult();
	        // und jetzt koennen wir das Ergebnis auch anzeigen
	        _resultFrame.setEnabled(true);
	        _resultFrame.setVisible(true);
	      }
	      else
	      {
    	    textFieldStatus.setText(queryFrameBundle.getString("textFieldStatus_text"));
	      }
	    }
	    catch (Throwable ex)
	    {
	      textFieldStatus.setText(queryFrameBundle.getString("textFieldStatus_text_1")+ex.getMessage());
	    }
	  }
	  finally
	  {
  	  buttonSuchen.setEnabled(true);
	  }
	}

  // Ergnisfenster wurde geschlossen
  public void resultFrameClosed(ResultFrame rf)
  {
    if (_resultFrame==rf)
    {
      _resultFrame=null;
    }
  }
  
	void buttonHilfe_ActionPerformed(java.awt.event.ActionEvent event)
	{
	  QueryFrameHelp frame=new QueryFrameHelp();
	  frame.setVisible(true);
	}

	class SymItem implements java.awt.event.ItemListener
	{
		public void itemStateChanged(java.awt.event.ItemEvent event)
		{
			Object object = event.getSource();
			if (object == choiceSuchart)
				choiceSuchart_ItemStateChanged(event);
		}
	}

  /**
    neue Suchart wurde ausgewaehlt
  */
	void choiceSuchart_ItemStateChanged(java.awt.event.ItemEvent event)
	{
	  String s=choiceSuchart.getSelectedItem();
	  QueryPanel p=(QueryPanel)_queries.get(s);
	  if (p==_oldPanel)
	    return;
	  // altes Panel verstecken
	  _oldPanel.setVisible(false);
	  _oldPanel.setEnabled(false);
	  // und neues anzeigen
	  p.setVisible(true);
	  p.setEnabled(true);
	  _oldPanel=p;
	  // Anpassung der Groesse erzwingen
    doLayout();
    panelPanels.doLayout();
    _oldPanel.doLayout();
	}

	void QueryFrame_WindowOpened(java.awt.event.WindowEvent event)
	{
	}
	
	Hashtable _queries=new Hashtable();
	QueryPanel _oldPanel=null;

	/**
	  neue Anfragemethode zur Liste hinzufuegen
	*/
	void addQueryMethod(QueryPanel panel)
	{
	  String name=panel.getQueryDescription();
	  _queries.put(name,panel);
	  choiceSuchart.addItem(name);
	  panelPanels.add(panel,name);
	  panel.setBounds(10,85,500,265);
	  // wenn es das erste ist, wird es auch angezeigt
	  if (1==choiceSuchart.getItemCount())
	  {
	    panel.setEnabled(true);
	    panel.setVisible(true);
	    choiceSuchart.select(name);
	    _oldPanel=panel;
	    // Groessenanpassung erzwingen
     doLayout();
     panelPanels.doLayout();
     _oldPanel.doLayout();
	  }
	  // ansonsten darf es nicht zu seheh sein
	  else
	  {
	    panel.setEnabled(false);
	    panel.setVisible(false);
	  }
	}
	
	int _resultNum=0;
	int _maxResultNum=0;

	// hier kommen die Callback-Methoden
	// damit koennen wir in der Statuszeile anzeigen, wie weit das Einlesen ist
  public void startResult()
  {
    textFieldStatus.setText(queryFrameBundle.getString("textFieldStatus_text_2"));
  }
  
  public void addResult(ResultDescriptor rd)
  {
    _resultNum++;
    textFieldStatus.setText(queryFrameBundle.getString("textFieldStatus_text_3")+_resultNum+"/"+_maxResultNum);
  }
  
  public void setResultNum(int num)
  {
    _maxResultNum=num;
  }
  
  public void endResult()
  {
    textFieldStatus.setText(queryFrameBundle.getString("textFieldStatus_text_4"));
    _resultNum=0;
  }
	static java.util.ResourceBundle queryFrameBundle = java.util.ResourceBundle.getBundle("de.hendriklipka.ldse.client.ui.QueryFrameBundle");
}
