/*
Copyright (C) 1999/2000 Hendrik Lipka.
All rights reserved.

This program is free software; you can redistribute it and/or modify it under the terms 
of the GNU General Public License as published by the Free Software Foundation; 
either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY 
WARRANTY; without even the implied warranty of MERCHANTABILITY or 
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License 
for more details. 

You should have received a copy of the GNU General Public License along with this 
program; if not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
Cambridge, MA 02139, USA or see http://www.gnu.org

If you encounter problems or have a suggestion, please mail to:
hendrik.lipka@gmx.de
*/

package de.hendriklipka.ldse.common;

import java.util.Hashtable;
import java.util.Date;
import java.util.StringTokenizer;
import java.io.PrintWriter;
import java.io.FileWriter;
import java.io.File;
import java.io.IOException;
import java.text.*;
import java.io.OutputStreamWriter;

import com.sun.xml.parser.Resolver;
import com.sun.xml.tree.XmlDocument;
import org.xml.sax.InputSource;
import org.w3c.dom.*;

public class Log implements de.hendriklipka.ldse.common.Updateable
{
  protected static int _nameLength=10;
  
  protected static Log _i=null;
  
  protected int _defLevel=0;
  protected Hashtable _groups=new Hashtable();
  protected boolean _writeTime=false;
  protected boolean _writeThread=false;
  protected boolean _writeGroup=false;
  protected boolean _writeArea=false;
  protected boolean _writeLevel=false;
  protected boolean _flush=false;
  
  protected PrintWriter _writer=null;
  
  protected DecimalFormat _format=new DecimalFormat("##");
  protected DateFormat _dateFormat=DateFormat.getDateTimeInstance(DateFormat.MEDIUM,DateFormat.MEDIUM);
  protected String _emptyString=null;
  
  class Area
  {
    protected int _level=0;
    public String _printName=null;
    public boolean _noscreen=true;
    public Area(int level,String name, boolean noscreen)
    {
      _level=level;
      _noscreen=noscreen;
      if (name.length()>_nameLength)
      {
        _printName=name.substring(0,_nameLength);
      }
      else
        _printName=name;
      while(_printName.length()<_nameLength)
      {
        _printName=_printName+" ";
      }
    }
  }
  
  class Group
  {
    protected int _level=0;
    protected Hashtable _areas=new Hashtable(); //keys=name, value=area 
    public String _printName=null;
    
    public Group(int level, String name)
    {
      _level=level;
      if (name.length()>_nameLength)
      {
        _printName=name.substring(0,_nameLength);
      }
      else
        _printName=name;
      while(_printName.length()<_nameLength)
      {
        _printName=_printName+" ";
      }
      Area a=new Area(level,"default",true);
    }
    
    public void addArea(String name, int level, boolean noscreen)
    {
      name=name.toLowerCase();
      Area a=new Area(level,name,noscreen);
      _areas.put(name,a);
    }
    
    public Area getArea(String name)
    {
      Object o=_areas.get(name);
      if (null==o)
      {
        return (Area)_areas.get("default");
      }
      Area a=(Area)o;
      return a;
    }
  }
  
  
  public static void init(Node configNode)
  {
    Log l=new Log(configNode);
    _i=l;
  }
  
  protected Log(Node configNode)
  {
    System.runFinalizersOnExit(true);
    
    boolean append=false;
    
    NamedNodeMap nnl=configNode.getAttributes();
    Node attr=nnl.getNamedItem("level");
    if (null!=attr)
    {
      String level=attr.getNodeValue();
      _defLevel=Integer.parseInt(level);
    }
    
    attr=nnl.getNamedItem("append");
    if (null!=attr)
    {
      String s=attr.getNodeValue().toLowerCase();
      if (s.equals("yes")||s.equals("ja")||s.equals("1")||s.equals("true"))
      {
        append=true;
      }
      else
      {
        append=false;
      }
    }
    
    String fname="log.txt";
    attr=nnl.getNamedItem("filename");
    if (null!=attr)
    {
      fname=attr.getNodeValue();
    }
    try
    {
      if (null==_i)
        _writer=new PrintWriter(new FileWriter(fname,append));
    }
    catch (java.io.IOException ex)
    {
      ex.printStackTrace(System.err);
    }
    
    attr=nnl.getNamedItem("flush");
    if (null!=attr)
    {
      String s=attr.getNodeValue().toLowerCase();
      if (s.equals("yes")||s.equals("ja")||s.equals("1")||s.equals("true"))
      {
        _flush=true;
      }
      else
      {
        _flush=false;
      }
    }
    
    attr=nnl.getNamedItem("writethread");
    if (null!=attr)
    {
      String s=attr.getNodeValue().toLowerCase();
      if (s.equals("yes")||s.equals("ja")||s.equals("1")||s.equals("true"))
      {
        _writeThread=true;
      }
      else
      {
        _writeThread=false;
      }
    }
    attr=nnl.getNamedItem("writetime");
    if (null!=attr)
    {
      String s=attr.getNodeValue().toLowerCase();
      if (s.equals("yes")||s.equals("ja")||s.equals("1")||s.equals("true"))
      {
        _writeTime=true;
      }
      else
      {
        _writeTime=false;
      }
    }
    attr=nnl.getNamedItem("writegroup");
    if (null!=attr)
    {
      String s=attr.getNodeValue().toLowerCase();
      if (s.equals("yes")||s.equals("ja")||s.equals("1")||s.equals("true"))
      {
        _writeGroup=true;
      }
      else
      {
        _writeGroup=false;
      }
    }
    attr=nnl.getNamedItem("writearea");
    if (null!=attr)
    {
      String s=attr.getNodeValue().toLowerCase();
      if (s.equals("yes")||s.equals("ja")||s.equals("1")||s.equals("true"))
      {
        _writeArea=true;
        _writeGroup=true;
      }
      else
      {
        _writeArea=false;
      }
    }
    attr=nnl.getNamedItem("writelevel");
    if (null!=attr)
    {
      String s=attr.getNodeValue().toLowerCase();
      if (s.equals("yes")||s.equals("ja")||s.equals("1")||s.equals("true"))
      {
        _writeLevel=true;
      }
      else
      {
        _writeLevel=false;
      }
    }
    attr=nnl.getNamedItem("namelen");
    if (null!=attr)
    {
      String s=attr.getNodeValue().toLowerCase();
      try
      {
        int i=Integer.parseInt(s);
        _nameLength=i;
      }
      catch (Exception ex) {}
    }
    NodeList nl=configNode.getChildNodes();
    // Gruppen einlesen
    for (int i=0;i<nl.getLength();i++)
    {
      Node node=nl.item(i);
      // <group> ?
      if ((node.getNodeType()==node.ELEMENT_NODE)&&(node.getNodeName().equals("group")))
      {
        int level=_defLevel;
        nnl=node.getAttributes();
        // level einlesen
        attr=nnl.getNamedItem("level");
        if (null!=attr)
        {
          String s=attr.getNodeValue();
          // parsen auf +/-
          if (s.startsWith("+"))
          {
            s=s.substring(1);
            level+=Integer.parseInt(s);
          }
          else if (s.startsWith("-"))
          {
            s=s.substring(1);
            level-=Integer.parseInt(s);
          }
          else
            level=Integer.parseInt(s);
        }
        // Namen einlesen
        attr=nnl.getNamedItem("name");
        if (null!=attr)
        {
          String s=attr.getNodeValue();
          s=s.toLowerCase();
          Group g=new Group(level,s);
          _groups.put(s,g);
          // Areas einlesen
          NodeList areas=node.getChildNodes();
          for (int j=0;j<areas.getLength();j++)
          {
            Node area=areas.item(j);
            //<area> ?
            if ((area.getNodeType()==area.ELEMENT_NODE)&&(area.getNodeName().equals("area")))
            {
              int aLevel=level;
              nnl=area.getAttributes();
              // level einlesen
              attr=nnl.getNamedItem("level");
              if (null!=attr)
              {
                s=attr.getNodeValue();
                // parsen auf +/-
                if (s.startsWith("+"))
                {
                  s=s.substring(1);
                  aLevel+=Integer.parseInt(s);
                }
                else if (s.startsWith("-"))
                {
                  s=s.substring(1);
                  aLevel-=Integer.parseInt(s);
                }
                else
                  aLevel=Integer.parseInt(s);
              }
              // noscreen einlesen
              boolean noscreen=true;
              attr=nnl.getNamedItem("noscreen");
              if (null!=attr)
              {
                s=attr.getNodeValue();
                if (s.equals("no"))
                  noscreen=false;
              }
              // Namen einlesen
              attr=nnl.getNamedItem("name");
              if (null!=attr)
              {
                s=attr.getNodeValue();
                g.addArea(s,aLevel,noscreen);
              } // if
            } // if (nodeType)
          } // for (areas)
        } // if (null!=attr)
      } // if
    } // for
    _emptyString=new String();
    for (int i=0;i<_nameLength;i++)
    {
      _emptyString=_emptyString+" ";
    }
  } // method Log()
  
  public static void log(String group, String area, int level, String msg)
  {
    try
    {
      if (null==msg)
        msg="!!! no message for logging";
      group=group.toLowerCase();
      area=area.toLowerCase();
      Group g=(Group)_i._groups.get(group);
      if (null==g)
      {
        log(level,msg);
        return;
      }
      Area a=g.getArea(area);
      if (null==a)
      {
        if (level<=g._level)
          _i.log(msg,g._printName,_i._emptyString,level,true);
      }
      else if(level<=a._level)
        _i.log(msg,g._printName,a._printName,level,a._noscreen);
    }
    catch (Throwable ex)
    {
      ex.printStackTrace(System.err);
    }
  }
  
  public static void log(String group, int level, String msg)
  {
    try
    {
      if (null==_i)
        return;
      if (null==msg)
        msg="!!! no message for logging";
      group=group.toLowerCase();
      Group g=(Group)_i._groups.get(group);
      if (null==g)
      {
        log(level,msg);
        return;
      }
      if (level<=g._level)
        _i.log(msg,g._printName,_i._emptyString,level,true);
    }
    catch (Throwable ex)
    {
      ex.printStackTrace(System.err);
    }
  }
  
  public static void log(int level, String msg)
  {
    try
    {
      if (null==_i)
        return;
      if (null==msg)
        msg="!!! no message for logging";
      if (level<=_i._defLevel)
      {
        _i.log(msg,_i._emptyString,_i._emptyString,level,true);
      }
    }
    catch (Throwable ex)
    {
      ex.printStackTrace(System.err);
    }
  }
  
  public static PrintWriter getWriter(String group, String area, int level)
  {
    try
    {
      if (null==_i)
        return new PrintWriter(new java.io.OutputStreamWriter(System.err));
      group=group.toLowerCase();
      area=area.toLowerCase();
      Group g=(Group)_i._groups.get(group);
      if (null==g)
      {
        System.err.println("no group for "+group);
        return getWriter(level);
      }
      Area a=g.getArea(area);
      if (null==a)
      {
        System.err.println("no area for "+group+":"+area);
        return getWriter(group,level);
      }
      if(level<=a._level)
        return _i.getLogWriter(g._printName,a._printName,level,a._noscreen);
      else 
        return _i.getNullWriter();
    }
    catch (Throwable ex)
    {
      ex.printStackTrace(System.err);
      return new PrintWriter(new OutputStreamWriter(System.err));
    }
  }
  
  public static PrintWriter getWriter(String group, int level)
  {
    try
    {
      if (null==_i)
        return new PrintWriter(new java.io.OutputStreamWriter(System.err));
      group=group.toLowerCase();
      Group g=(Group)_i._groups.get(group);
      if (null==g)
        return getWriter(level);
      if (level<=g._level)
        return _i.getLogWriter(g._printName,_i._emptyString,level,true);
      else 
        return _i.getNullWriter();
    }
    catch (Throwable ex)
    {
      ex.printStackTrace(System.err);
      return new PrintWriter(new OutputStreamWriter(System.err));
    }
  }
  
  public static PrintWriter getWriter(int level)
  {
    try
    {
      if (null==_i)
        return new PrintWriter(new java.io.OutputStreamWriter(System.err));
      if (level<=_i._defLevel)
        return _i.getLogWriter(_i._emptyString,_i._emptyString,level,true);
      else 
        return _i.getNullWriter();
    }
    catch (Throwable ex)
    {
      ex.printStackTrace(System.err);
      return new PrintWriter(new OutputStreamWriter(System.err));
    }
  }
  
  protected String doPrefix(String group, String area, int level)
  {
    StringBuffer sb=new StringBuffer();
    String s=" ";
    if (_writeTime)
    {
      sb.append(_dateFormat.format(new Date(System.currentTimeMillis())));
      sb.append(s);
    }
    if (_writeThread)
    {
      sb.append(Thread.currentThread().getName());
      sb.append(s);
    }
    if (_writeGroup)
      sb.append("["+group+"]");
    if (_writeArea)
      sb.append("["+area+"]");
    if (_writeLevel)
      sb.append("["+_format.format(level)+"]");
    if(s.length()>0)
    {
      sb.append(":");
    }
    return sb.toString();
  }
  
  protected synchronized PrintWriter getLogWriter(String group, String area, int level,boolean noscreen)
  {
    String s=doPrefix(group,area,level);
    int l=s.length();
    StringBuffer sb=new StringBuffer(l);
    for (int i=0;i<l;i++)
    {
      sb.append(' ');
    }
    String pre=sb.toString()+">";
    _writer.print(s);
    return new ErrorWriter(_writer,l,pre);
  }
  
  protected PrintWriter getNullWriter()
  {
    return new NullWriter();
  }
  
  protected synchronized void log(String msg, String group, String area, int level, boolean noscreen)
  {
    if (null==msg)
    msg="!!! no message for logging";
    String s=doPrefix(group,area,level);
    
    // Test, ob mehrere Zeilen...
    if (-1!=msg.indexOf("\n"))
    {
      StringTokenizer st=new StringTokenizer(msg,"\n");
      int l=s.length();
      StringBuffer sb=new StringBuffer(l);
      for (int i=0;i<l-1;i++)
      {
        sb.append(' ');
      }
      String pre=sb.toString()+">";
      int i=0;
      while (st.hasMoreElements())
      {
        String line=(String)st.nextElement();
        if (0!=i++)
          s+=pre;
        s+=line+"\n";
      }
      _writer.print(s);
      if (!noscreen)
        System.err.print(s);
    }
    else
    {
      _writer.println(s+msg);
      if (!noscreen)
        System.err.println(s+msg);
    }
    
    // bei Bedarf flushen
    if (_flush)
    {
      _writer.flush();
      System.err.flush();
    }
  }
  
  public static void error(String group, String area, int level, String msg)
  {
    if (null==msg)
    msg="!!! no message for logging";
    if (null==_i)
    {
      System.err.println("fatal error: "+msg+"\nexiting");
    }
    _i.log("fatal error: "+msg+"\nexiting",group,area,level,false);
    _i._writer.flush();
    System.err.flush();
    System.exit(1);
  }
  
  protected void finalize()
  {
    try
    {
      if (null!=_writer)
      {
        synchronized(_writer)
        {
          if (null!=_writer)
          {
            _writer.flush();
            _writer.close();
            _writer=null;
          }
        }
      }
    }
    catch (Throwable ex)
    {
      ex.printStackTrace(System.err);
    }
  }

  public static Log getInstance()
  {
    return _i;
  }
  
  public void update(Node configNode)
  {
    if (null==configNode)
    {
      Log.log(0,"error while updating debug config: no config node...");
      return;
    }
    Log l=new Log(configNode);
    _defLevel=0;
    _i._groups=l._groups;
    _i._writeTime=l._writeTime;
    _i._writeThread=l._writeThread;
    _i._writeGroup=l._writeGroup;
    _i._writeArea=l._writeArea;
    _i._writeLevel=l._writeLevel;
    _i._flush=l._flush;
  }
  
  public static void main(String[] args) 
  {
    try
    {
  	  InputSource	input = Resolver.createInputSource (new File ("logtest.xml"));
	    XmlDocument	doc = XmlDocument.createXmlDocument (input, false);
      Element el=doc.getDocumentElement();
      NodeList cnl=el.getElementsByTagName("debug");
      Node configNode=cnl.item(0);
      init(configNode);
      _i._writeTime=true;
      _i._writeThread=true;
      _i._writeGroup=true;
      _i._writeArea=true;
      _i._writeLevel=true;
      _i._flush=true;
      long start=System.currentTimeMillis();
      for (int i=0;i<10000;i++)
      {
        int l=i%5;
        Log.log("spider","",l,"das ist ein text");
        Log.log("spider","error",l,"das ist ein text");
        Log.log("spider","info",l,"das ist ein text");
        Log.log("spider","url",l,"das ist ein text");
        Log.log("indexer","",l,"das ist ein text");
        Log.log("indexer","config",l,"das ist ein text");
        Log.log("indexer","error",l,"das ist ein text");
        Log.log("indexer","document",l,"das ist ein text");
        Log.log("dataclient","",l,"das ist ein text");
        Log.log("dataclient","error",l,"das ist ein text");
        Log.log("dataclient","config",l,"das ist ein text");
        Log.log("dataclient","command",l,"das ist ein text");
        Log.log("dataserver","",l,"das ist ein text");
        Log.log("dataserver","config",l,"das ist ein text");
        Log.log("dataserver","error",l,"das ist ein text");
        Log.log("dataserver","net",l,"das ist ein text");
        Log.log("database","",l,"das ist ein text");
        Log.log("database","config",l,"das ist ein text");
        Log.log("database","error",l,"das ist ein text");
        Log.log("database","sql",l,"das ist ein text");
        Log.log("database","startup",l,"das ist ein text");
        Log.log("brokerserver","",l,"das ist ein text");
        Log.log("brokerserver","config",l,"das ist ein text");
        Log.log("brokerserver","error",l,"das ist ein text");
        Log.log("brokerserver","net",l,"das ist ein text");
        Log.log("queryserver","",l,"das ist ein text");
        Log.log("queryserver","config",l,"das ist ein text");
        Log.log("queryserver","error",l,"das ist ein text");
        Log.log("queryserver","query",l,"das ist ein text");
        Log.log("queryserver","net",l,"das ist ein text");
        Log.log("queryserver","statistic",l,"das ist ein text");
        Log.log("server","",l,"das ist ein text");
        Log.log("server","config",l,"das ist ein text");
        Log.log("server","error",l,"das ist ein text");
        Log.log("server","threads",l,"das ist ein text");
        Log.log("document","",l,"das ist ein text");
        Log.log("document","error",l,"das ist ein text");
        Log.log("document","info",l,"das ist ein text");
        Log.log("document","data",l,"das ist ein text");
        Log.log("","",l,"das ist ein text");
      }
      long ende=System.currentTimeMillis();
      long time=(ende-start);
      System.out.println("1. Zeit (ms) : "+time+" -> "+((double)time)/((double)400000)+" per log");
    }
    catch (Throwable ex)
    {
      ex.printStackTrace();
    }
  }
}

  class NullWriter extends PrintWriter
  {
    public NullWriter()
    {
      super((PrintWriter)null);
    }
    public void write(int c) {
    }
    public void write(char buf[], int off, int len) {
    }
    public void write(char buf[]) {
    }
    public void write(String s, int off, int len) {
    }
    public void write(String s) {
    }
    public void print(boolean b) {
    }
    public void print(char c) {
    }
    public void print(int i) {
    }
    public void print(long l) {
    }
    public void print(float f) {
    }
    public void print(double d) {
    }
    public void print(char s[]) {
    }
    public void print(String s) {
    }
    public void print(Object obj) {
    }
    public void println() {
    }
    public void println(boolean x) {
    }
    public void println(char x) {
    }
    public void println(int x) {
    }
    public void println(long x) {
    }
    public void println(float x) {
    }
    public void println(double x) {
    }
    public void println(char x[]) {
    }
    public void println(String x) {
    }
    public void println(Object x) {
    }
  }
  
  class ErrorWriter extends PrintWriter
  {
    protected String _pre=null;
    protected int _len=0;
    protected boolean _doPrefix=false;
    
    public ErrorWriter(PrintWriter w, int len,String s)
    {
      super(w);
      _pre=s;
      _len=len;
    }
    
    private void myNewLine() 
    {
      super.println();
      _doPrefix=true;
    }
    
    private void myNewLine2()
    {
      if ((_len>0)&&_doPrefix)
        print(_pre);
      _doPrefix=false;
    }
    
    public void println() 
    {
      synchronized (lock) 
      {
        myNewLine2();
        myNewLine();
      }
    }

    public void println(boolean x) 
    {
      synchronized (lock) 
      {
        myNewLine2();
        print(x);
        myNewLine();
      }
    }

    public void println(char x) 
    {
      synchronized (lock) 
      {
        myNewLine2();
        print(x);
        myNewLine();
      }
    }

    public void println(int x) 
    {
      synchronized (lock) 
      {
        myNewLine2();
        print(x);
        myNewLine();
      }
    }

    public void println(long x) 
    {
      synchronized (lock) 
      {
        myNewLine2();
        print(x);
        myNewLine();
      }
    }

    public void println(float x) 
    {
      synchronized (lock) 
      {
        myNewLine2();
        print(x);
        myNewLine();
      }
    }

    public void println(double x) 
    {
      synchronized (lock) 
      {
        myNewLine2();
        print(x);
        myNewLine();
      }
    }

    public void println(char x[]) 
    {
      synchronized (lock) 
      {
        myNewLine2();
        print(x);
        myNewLine();
      }
    }

    public void println(String x) 
    {
      synchronized (lock) 
      {
        myNewLine2();
        print(x);
        myNewLine();
      }
    }

    public void println(Object x) 
    {
      synchronized (lock) 
      {
        myNewLine2();
        print(x);
        myNewLine();
      }
    }
  }

// File fuer den Performance-Test (logtest.xml)
/*
	<debug level="2" writetime="no" writegroup="yes" writearea="yes" writelevel="no" writethread="no" flush="yes" filename="logtest.txt" namelen="12">
		<group name="spider">
			<area name="error" level="3"/>
			<area name="info" level="1"/>
			<area name="url" level="2"/>
			<area name="config" level="1"/>
			<area name="threads" level="2"/>
		</group>
		<group name="indexer" level="2">
			<area name="config" level="2"/>
			<area name="error" level="2"/>
			<area name="document" level="2"/>
		</group>
		<group name="dataclient">
			<area name="error" level="2"/>
			<area name="config" level="1"/>
			<area name="command" level="2"/>
		</group>
		<group name="server">
			<area name="error" level="2" noscreen="yes"/>
			<area name="config" level="2" noscreen="yes"/>
			<area name="threads" level="2" noscreen="yes"/>
		</group>
		<group name="document">
			<area name="error" level="2"/>
			<area name="info" level="2"/>
			<area name="data" level="2"/>
		</group>
		<group name="database">
			<area name="config" level="1"/>
			<area name="error" level="2" noscreen="no"/>
			<area name="startup" level="2" noscreen="no"/>
			<area name="query" level="2" noscreen="no"/>
			<area name="sql" level="2"/>
			<area name="statistic" level="2"/>
		</group>
		<group name="dataserver">
			<area name="config" level="2"/>
			<area name="error" level="2"/>
			<area name="statistic" level="1"/>
			<area name="query" level="2"/>
			<area name="net" level="2"/>
		</group>
		<group name="queryserver" level="2">
			<area name="error" level="2"/>
			<area name="config" level="2"/>
			<area name="net" level="2"/>
			<area name="statistic" level="2"/>
			<area name="query" level="2"/>
		</group>
		<group name="brokerserver">
			<area name="config" level="+3"/>
			<area name="error" level="2"/>
			<area name="net" level="1"/>
			<area name="statistic" level="1"/>
			<area name="query" level="1"/>
		</group>
		<group name="lock">
			<area name="trace" level="0"/>
		</group>
	</debug>

*/