/*
Copyright (C) 1999/2000 Hendrik Lipka.
All rights reserved.

This program is free software; you can redistribute it and/or modify it under the terms 
of the GNU General Public License as published by the Free Software Foundation; 
either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY 
WARRANTY; without even the implied warranty of MERCHANTABILITY or 
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License 
for more details. 

You should have received a copy of the GNU General Public License along with this 
program; if not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
Cambridge, MA 02139, USA or see http://www.gnu.org

If you encounter problems or have a suggestion, please mail to:
hendrik.lipka@gmx.de
*/

package de.hendriklipka.ldse.server.broker;

import java.net.*;
import java.io.*;
import java.util.Vector;
import java.util.Enumeration;

import org.w3c.dom.*;

import de.hendriklipka.ldse.server.data.document.*;
import de.hendriklipka.ldse.server.data.server.TripletEntry;
import de.hendriklipka.ldse.server.data.server.ServerDescriptor;
import de.hendriklipka.ldse.server.dataserver.DBConnection;
import de.hendriklipka.ldse.server.broker.BrokerQuery;
import de.hendriklipka.ldse.common.Log;

/**

*/
public class BrokerServer implements de.hendriklipka.ldse.common.Updateable, de.hendriklipka.ldse.common.StatisticWriter
{
  protected DBConnection _con=null;
  
  protected static long _queries=0;
  protected static long _subServers=0;
  protected static long _local=0;
  static long _resultDocs=0;

  public BrokerServer(DBConnection con,Node configNode)
  {
    _con=con;
  }
  
  // lokale Abfrage durchfuehren
  // entscheiden, welche subServer gefragt werden
  // entsprechende BrokerConnections zusammenstellen
  // !! es muss mindestens eine BC geliefert werden!!
  public BrokerConnection[] getConnections(BrokerQuery bq)
  {
    ServerDescriptor[] sda=_con.queryServers(bq.getTriplets());
    int l=sda.length;
    double[] maxData=new double[l];
    double[] sumData=new double[l];
    double[] docData=new double[l];
    double maxSum=0;
    double sumSum=0;
    double docSum=0;
    
    for (int i=0;i<l;i++)
    {
      double level=0;
      double maxLevel=0;
      double docs=0;
      TripletList list=sda[i].getTripletList();
      Enumeration e=bq.getTriplets().getTriplets();
      // alle gesuchten Triplets durchgehen
      while (e.hasMoreElements())
      {
        Triplet tr=(Triplet)e.nextElement();
        // schauen, ob dieses Dokument es hat
        TripletEntry tr2=(TripletEntry)list.getTriplet(tr);
        // ja -> Relevanz entsprechend erhoehen
        if (null!=tr2)
        {
          level+=(tr.count*tr2.count);
          maxLevel+=(tr.count*tr2.getMaxDocNum());
          docs+=tr2.getDocCount();
        }
      }
      // Summen pro Dokument speichern und weiter aufsummieren
      sumData[i]=level;
      sumSum+=level;
      maxData[i]=maxLevel;
      maxSum+=maxLevel;
      docSum+=docs;
      docData[i]=docs;
    }
    
    // Durchschnitt ueber Server berechnen
    double sumAvg=sumSum/l;
    double maxAvg=maxSum/l;
    double docAvg=docSum/l;

    // zaehlen, wieviele Server wir fragen
    int resCount=0;
    for (int i=0;i<sda.length;i++)
    {
      if ((sumData[i]>=sumAvg)||(maxData[i]>=maxAvg)||(docData[i]>=docAvg))
      {
        resCount++;
      }
    }

    BrokerConnection[] bca=new BrokerConnection[resCount];
    resCount=0;
    for (int i=0;i<sda.length;i++)
    {
      try
      {
        if ((sumData[i]<sumAvg)&&(maxData[i]<maxAvg)&&(docData[i]<docAvg))
        {
          Log.log("broker","",1,"skipping server "+sda[i].getID());
          continue;
        }
        if (0==sda[i].getID())
        {
          Log.log("broker","",1,""+resCount+": choosing local server");
          bca[resCount++]=new BrokerConnection(_con,bq);
          _local++;
        }
        else
        {
          Log.log("broker","",1,""+resCount+": choosing remote server: "+sda[i].getName());
          bca[resCount++]=new BrokerConnection(sda[i].getName(),sda[i].getPort(),bq);
          _subServers++;
        }
      }
      catch (Exception ex)
      {
        ex.printStackTrace();
      }
    }
    // wenn wir keinen gewaehlt haben, fragen wir lokal nach
    // im Zweifelsfalle ist da nix, so dass auch kein Ergebnis kommt...
    // aber eigentlich sollte immer ein Wert ueber dem Durchschnitt sein
    // (oder zumindest gleich damit...)
    // macht sich aber fuer Tests gut, da dadurch kein ServerDescriptor berechnet werden muss
    if (0==bca.length)
    {
      bca=new BrokerConnection[1];
      bca[0]=new BrokerConnection(_con,bq);
    }
    return bca;
  }
  
  public void update(Node configNode)
  {
  }
  
  public void writeStatistic(PrintWriter pw) throws IOException
  {
    pw.println("*** BrokerServer");
    pw.println("queries:           "+_queries);
    pw.println("used subservers:   "+_subServers);
    pw.println("used local server: "+_local);
    pw.println("resulting docs:    "+_resultDocs);
    pw.println();
  }
}
