/*
Copyright (C) 1999/2000 Hendrik Lipka.
All rights reserved.

This program is free software; you can redistribute it and/or modify it under the terms 
of the GNU General Public License as published by the Free Software Foundation; 
either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY 
WARRANTY; without even the implied warranty of MERCHANTABILITY or 
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License 
for more details. 

You should have received a copy of the GNU General Public License along with this 
program; if not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
Cambridge, MA 02139, USA or see http://www.gnu.org

If you encounter problems or have a suggestion, please mail to:
hendrik.lipka@gmx.de
*/

package de.hendriklipka.ldse.server.data.document;

import java.io.*;
import java.net.URL;
import java.util.zip.GZIPInputStream;

import de.hendriklipka.ldse.common.Log;
import de.hendriklipka.ldse.server.data.document.Document;
import de.hendriklipka.ldse.server.indexer.DocumentFactory;
import de.hendriklipka.ldse.server.data.URLInfo;
import de.hendriklipka.ldse.common.Spider;

/**
  zerlegt ein GZip-Dokument in Worte
  aus dem Namen der Datei wird der originale Namen bestimmt und versucht, dafuer
  ein Document zu erzeugen
  geht das, wird der Stream entpackt und umgeleitet
*/
public class GZipDocument extends Document implements Spider
{
  protected Reader _reader=null;
  protected InputStream _iStream=null;
  protected Document _doc=null;
  protected URL _url=null;
  
  public boolean init()
  {
		try
		{
		  _iStream=new GZIPInputStream(_urlInfo.getInputStreamForURL(_urlInfo._url));
		  _reader=new BufferedReader(new InputStreamReader(_iStream));

		  String n=_urlInfo._url.getFile();
		  int pos=n.lastIndexOf('.');
		  n=n.substring(0,pos);
  		
      Log.log("indexer","document",2,"gzip-document: new file name is "+n);
      URL u=new URL(_urlInfo._url.getProtocol(),_urlInfo._url.getHost(),_urlInfo._url.getPort(),n);
      URLInfo ui=new URLInfo();
      ui._url=u;
      ui._contextURL=_urlInfo._contextURL;
      ui._urlStr=u.toString();
      ui._spiderID=_urlInfo._spiderID;
      ui._spider=this;
		  ui._type=null;
		  // damit versorgen wir das neue Dokument mit den richtigen Daten, da wir selber
		  // ein Spider sind...
		  _url=u;
		  
		  _doc=DocumentFactory.getDocument(ui);
		  if (null!=_doc)
  		  return _doc.init();
      Log.log("indexer","document",2,"gzip-document: no document");
		}
		catch (Exception ex)
		{
		  _doc=null;
		  return false;
		}
    return false;
  }
  
  public boolean hasMoreWords()
  {
    if (null==_doc)
    {
      return false;
    }
    return _doc.hasMoreWords();
  }
  
  public String nextWord()
  {
    if (null==_doc)
    {
      return null;
    }
    return _doc.nextWord();
  }
  
  public String getDescriptiveWords()
  {
    if (null==_doc)
    {
      return "";
    }
    return _doc.getDescriptiveWords();
  }
  
  public void finish()
  {
    try
    {
      _doc.finish();
    }
    catch (Exception ex) {}
  }
  
  public java.io.Reader getReaderForURL(java.net.URL url) throws java.io.IOException
  {
    if (url==_url)
    {
      return _reader;
    }
    return null;
  }
  
  public java.io.InputStream getInputStreamForURL(java.net.URL url) throws java.io.IOException
  {
    if (url==_url)
    {
      return _iStream;
    }
    return null;
  }
  
  public void init(org.w3c.dom.Node configNode, int id){}
  public boolean hasMoreElements() { return false;}
  public URLInfo getNextElement() {return null;}
  public void start() {}
}