/*
Copyright (C) 1999/2000 Hendrik Lipka.
All rights reserved.

This program is free software; you can redistribute it and/or modify it under the terms 
of the GNU General Public License as published by the Free Software Foundation; 
either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY 
WARRANTY; without even the implied warranty of MERCHANTABILITY or 
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License 
for more details. 

You should have received a copy of the GNU General Public License along with this 
program; if not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
Cambridge, MA 02139, USA or see http://www.gnu.org

If you encounter problems or have a suggestion, please mail to:
hendrik.lipka@gmx.de
*/

package de.hendriklipka.ldse.server.query;

import java.net.Socket;
import java.io.*;
import java.util.Vector;

import com.objectspace.jgl.Container;
import com.objectspace.jgl.SList;
import com.objectspace.jgl.ForwardIterator;
import com.objectspace.jgl.OrderedSet;
import com.objectspace.jgl.BinaryPredicate;

import com.sun.xml.tree.XmlDocument;
import org.xml.sax.InputSource;
import org.w3c.dom.*;

import de.hendriklipka.ldse.common.Log;
import de.hendriklipka.ldse.common.HTMLCommentFilterReader;
import de.hendriklipka.ldse.common.ResultDescriptor;
import de.hendriklipka.ldse.server.broker.*;
import de.hendriklipka.ldse.server.dataserver.DBConnection;

/**
  Behandelt Verbindung zu einem Frage-Client
*/
public class ClientConnection extends Thread
{
  protected Socket _socket=null;
  protected QueryFilterReader _reader=null;
  protected PrintWriter _writer=null;
  
  protected DBConnection _con;
  protected BrokerServer _brokerServer=null;
  
  public ClientConnection(Socket socket, DBConnection con, BrokerServer brokerServer)
  {
    super("ClientConnection");
    _con=con;
    _socket=socket;
    _brokerServer=brokerServer;
    try
    {
      InputStream s1=_socket.getInputStream();
      Reader r1=new InputStreamReader(s1);
      Reader r2=new HTMLCommentFilterReader(r1);
      _reader=new QueryFilterReader(r2);
      _writer=new PrintWriter(new OutputStreamWriter(_socket.getOutputStream()));
    }
    catch (Throwable ex)
    {
      ex.printStackTrace(Log.getWriter("queryserver","error",0));
    }
  }
  
  public void run()
  {
    try
    {
      long start=System.currentTimeMillis();
      // XML-Parser erzeugen
      InputSource input=new InputSource(_reader);
	    XmlDocument	doc = XmlDocument.createXmlDocument (input, false);
      StringWriter sw=new StringWriter();
      org.w3c.dom.Node n=doc.getDocumentElement();
      // Anfrage einlesen
      doc.write(sw);
      sw.flush();
      Log.log("queryserver","query",1,sw.toString());
      // passenden Processor erzeugen
      QueryProcessor qp=QueryProcessorFactory.getQueryProcessor(doc);
      Element el=doc.getDocumentElement();

      // Limits extrahieren
      QueryLimits ql=null;
      NodeList cnl=el.getElementsByTagName("limit");
      if (null!=cnl)
      if(0!=cnl.getLength())
      {
        Node node=cnl.item(0);
        ql=new QueryLimits(node);
        qp.setLimits(ql);
      }

      // Processor initialisieren
      qp.init();
      qp.preprocessQuery();

      // Brokering
      BrokerQuery bq=qp.createBrokerQuery();
      BrokerConnection[] bca=_brokerServer.getConnections(bq);
      BrokerThread[] bta=new BrokerThread[bca.length];
      // eigener Thread fuer jede Anfrage
      for (int i=0;i<bca.length;i++)
      {
        BrokerThread bt=new BrokerThread(bca[i]);
        bta[i]=bt;
        bt.start();
      }
      // warten auf die Threads und Ergebnisse holen
      Container[] ca=new Container[bca.length];
      for (int i=0;i<bca.length;i++)
      {
        bta[i].join();
        Container rda=bta[i].getResult();
        ca[i]=rda;
        if (null!=rda)
          Log.log("queryserver","query",1,"got result from BrokerConnection: "+rda.size());
      }

      // Results zusammenfassen
      Container result=null;
      if (null!=ql)
      {
        if (!ql._sorted&&(0==ql._maxHits))
        {
          result=new SList();
        }
      }
      if (null==result)
      {
        result=new OrderedSet(ResultDescriptor.getPredicate(),true);
      }
      for (int i=0;i<ca.length;i++)
      {
        Container rda=ca[i];
        if (null!=rda)
        {
          ForwardIterator it=rda.start();
          while (!it.atEnd())
          {
            ResultDescriptor rd=(ResultDescriptor)it.get();
            result.add(rd);
            it.advance();
          }
        }
        else
        {
          Log.log("queryserver","query",1,"no container...");
        }
      }

      Log.log("queryserver","query",1,"result: "+result.size()+" elements");
      // Limits fuer maxCount beachten
      int maxNum=result.size();
      if (null!=ql)
      {
        if (0!=ql._maxHits)
          if(ql._maxHits<maxNum)
            maxNum=ql._maxHits;
      }
      // Ergebnis ausgeben
      Log.log("queryserver","query",2,"printing result to client < "+maxNum);
      _writer.println("<queryresult count=\""+maxNum+"\">");
      _writer.flush();
      ForwardIterator it=result.start();
      int i=0;
      while(!it.atEnd()&&0<maxNum)
      {
        // jeder ResultDescriptor kann sich selbst ausgeben
        ResultDescriptor rd=(ResultDescriptor)it.get();
        rd.writeAsXML(_writer);
        it.nextElement();
        maxNum--;
        i++;
      }
      _writer.println("</queryresult>");
      _writer.flush();
      Log.log("queryserver","query",3,"printed result to client: "+i);
      long end=System.currentTimeMillis();
      QueryServer._time+=(end-start);
      QueryServer._resultDocs+=i;
    }
    catch (Throwable ex)
    {
      ex.printStackTrace(Log.getWriter("queryserver","error",0));
      ex.printStackTrace();
    }
    finally
    {
      try
      {
        Log.log("queryserver","query",1,"calling close's");
        _writer.flush();
        _reader.close2();
        _writer.close();
        _socket.close();
      }
      catch (IOException ex) 
      {
      ex.printStackTrace(Log.getWriter("queryserver","error",0));
      }
    }
  }
  
  private class BrokerThread extends Thread
  {
    protected BrokerConnection _bc=null;
    protected Container _result=null;
    
    public BrokerThread(BrokerConnection bc)
    {
      _bc=bc;
    }
    
    public void run()
    {
      _bc.doQuery();
      _result=_bc.getResult();
    }
    
    public Container getResult()
    {
      return _result;
    }
  }
}