/*
Copyright (C) 1999/2000 Hendrik Lipka.
All rights reserved.

This program is free software; you can redistribute it and/or modify it under the terms 
of the GNU General Public License as published by the Free Software Foundation; 
either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY 
WARRANTY; without even the implied warranty of MERCHANTABILITY or 
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License 
for more details. 

You should have received a copy of the GNU General Public License along with this 
program; if not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
Cambridge, MA 02139, USA or see http://www.gnu.org

If you encounter problems or have a suggestion, please mail to:
hendrik.lipka@gmx.de
*/

package de.hendriklipka.ldse.server.query;

import java.io.*;

import de.hendriklipka.ldse.common.Log;

/**
  filtert aus einem Reader die fuer eine Query noetigen XML-Daten heraus
  dies betrifft alles zwischen <query> und </query>
  auf das Starttag (<query>) qird nicht getestet
  nach dem Endetag (</query>) wird -1 fuer Streamende geliefert
  es wird korrekt auf <![CDATA[ und ]]> geachtet, und entwertete Tags zu erkennen
  Kommentare sollten mit dem HTMLFilterReader behandelt werden
*/
public class QueryFilterReader extends java.io.FilterReader
{
  protected boolean _queryFinished=false;
  protected boolean _inCData=false;
  protected int _state=NORMAL;
  protected final static int NORMAL=0;
  protected final static int QUOTE_IN1=1;   // <
  protected final static int CDATA_IN2=2;   // !
  protected final static int CDATA_IN3=3;   // [
  protected final static int CDATA_IN4=4;   // C
  protected final static int CDATA_IN5=5;   // D
  protected final static int CDATA_IN6=6;   // A
  protected final static int CDATA_IN7=7;   // T
  protected final static int CDATA_IN8=8;   // A
  protected final static int CDATA_OUT1=9;  // ]
  protected final static int CDATA_OUT2=10; // ]
  protected final static int QUERY_OUT2=11; // /
  protected final static int QUERY_OUT3=12; // q
  protected final static int QUERY_OUT4=13; // u
  protected final static int QUERY_OUT5=14; // e
  protected final static int QUERY_OUT6=15; // r
  protected final static int QUERY_OUT7=16; // y
  
	public QueryFilterReader(Reader in)
	{
		super(in);
	}
	
  /**
    * Read characters into a portion of an array.
    * does the filtering
    * @exception  IOException  If an I/O error occurs
  */
  public int read(char cbuf[], int off, int len) throws IOException 
  {
    if (_queryFinished)
      return -1;
    int r=len;
    for (int i=off;i<off+len;i++)
    {
      int c=in.read();
      if (-1==c)
      {
        _queryFinished=true;
        r=i-off;
        break;
      }
      cbuf[i]=(char)c;
      parseChar((char)c);
      if (_queryFinished)
      {
        _queryFinished=true;
        r=i-off+1;
        break;
      }
    }
    return r;
  }
  
  /**
    Parsing des Input
    realisiert als Statemachine
  */
  protected void parseChar(char c)
  {
    switch (_state)
    {
      case NORMAL: if ('<'==c)
                     _state=QUOTE_IN1;
                   else if (']'==c)
                     _state=CDATA_OUT1;
                   break;
      case QUOTE_IN1: if ('!'==c)
                        _state=CDATA_IN2;
                      else if ((!_inCData)&&('/'==c))
                        _state=QUERY_OUT2;
                      else
                        _state=NORMAL;
                      break;
      case CDATA_IN2: if ('['==c)
                        _state=CDATA_IN3;
                      else
                        _state=NORMAL;
                      break;
      case CDATA_IN3: if('C'==c)
                        _state=CDATA_IN4;
                      else
                        _state=NORMAL;
                      break;
      case CDATA_IN4: if ('D'==c)
                        _state=CDATA_IN5;
                      else
                        _state=NORMAL;
                      break;
      case CDATA_IN5: if ('A'==c)
                        _state=CDATA_IN6;
                      else
                        _state=NORMAL;
                      break;
      case CDATA_IN6: if ('T'==c)
                        _state=CDATA_IN7;
                      else
                        _state=NORMAL;
                      break;
      case CDATA_IN7: if ('A'==c)
                        _state=CDATA_IN8;
                      else
                        _state=NORMAL;
                      break;
      case CDATA_IN8: if('['==c)
                        _inCData=true;
                      _state=NORMAL;
                      break;
      case CDATA_OUT1: if (']'==c)
                         _state=CDATA_OUT2;
                       else
                         _state=NORMAL;
                       break;
      case CDATA_OUT2: if ('>'==c)
                         _inCData=false;
                       _state=NORMAL;
                       break;
      case QUERY_OUT2: if ('q'==c)
                         _state=QUERY_OUT3;
                       else
                         _state=NORMAL;
                       break;
      case QUERY_OUT3: if ('u'==c)
                         _state=QUERY_OUT4;
                       else
                         _state=NORMAL;
                       break;
      case QUERY_OUT4: if ('e'==c)
                         _state=QUERY_OUT5;
                       else
                         _state=NORMAL;
                       break;
      case QUERY_OUT5: if ('r'==c)
                         _state=QUERY_OUT6;
                       else
                         _state=NORMAL;
                       break;
      case QUERY_OUT6: if ('y'==c)
                         _state=QUERY_OUT7;
                       else
                         _state=NORMAL;
                       break;
      case QUERY_OUT7: if ('>'==c)
                       {
                         _queryFinished=true;
                       }
                       else
                         _state=NORMAL;
                       break;
    }
  }

  /**
    * Read a single character.  This method will block until a character is
    * available, an I/O error occurs, or the end of the stream is reached.
    *
    * <p> Subclasses that intend to support efficient single-character input
    * should override this method.
    *
    * @return     The character read, as an integer in the range 0 to 16383
    *             (<tt>0x00-0xffff</tt>), or -1 if the end of the stream has
    *             been reached
    *
    * @exception  IOException  If an I/O error occurs
  */
  public int read() throws IOException 
  {
    if (_queryFinished)
      return -1;
    char cb[] = new char[1];
    if (read(cb, 0, 1) == -1)
      return -1;
    else
      return cb[0];
  }

  /**
    * Read characters into an array.  This method will block until some input
    * is available, an I/O error occurs, or the end of the stream is reached.
    *
    * @param       cbuf  Destination buffer
    *
    * @return      The number of bytes read, or -1 if the end of the stream
    *              has been reached
    *
    * @exception   IOException  If an I/O error occurs
  */
  public int read(char cbuf[]) throws IOException 
  {
    if (_queryFinished)
      return -1;
    return read(cbuf, 0, cbuf.length);
  }

  /**
    * Skip characters.  This method will block until some characters are
    * available, an I/O error occurs, or the end of the stream is reached.
    *
    * @param  n  The number of characters to skip
    *
    * @return    The number of characters actually skipped
    *
    * @exception  IOException  If an I/O error occurs
  */
  private static final int maxSkipBufferSize = 8192;
  private char skipBuffer[] = null;
  public long skip(long n) throws IOException 
  {
    int nn = (int) Math.min(n, maxSkipBufferSize);
    synchronized (lock) 
    {
      if ((skipBuffer == null) || (skipBuffer.length < nn))
        skipBuffer = new char[nn];
      long r = n;
      while (r > 0) 
      {
        int nc = read(skipBuffer, 0, nn);
        if (nc == -1)
          break;
        r -= nc;
      }
      return n - r;
    }
  }
    
  /**
    * Tell whether this stream is ready to be read.
    *
    * @exception  IOException  If an I/O error occurs
  */
  public boolean ready() throws IOException 
  {
    if (_queryFinished)
      return false;
    return in.ready();
  }
  
  public void close() throws IOException 
  { // nichts tun, weil wir den Stream noch brauchen...
    Log.log("queryserver","query",4,"QueryFilterReader.close()");
  }
  
  public void close2() throws IOException 
  {
    super.close();
  }
  
  public static void main(String[] args)
  {
    String[] tests=
    {
      "<query></query>",
      "<query></query>unsinn",
      "<query>kjahjasdg</query>",
      "<query>jkasdgjkds</query>unsinn",
      "<query>kjdfh<![CDATA[jherg]]></query>",
      "<query>kjdfh<![CDATA[jherg]]></query>unsinn",
      "<query>kjdfh<![CDATA[jh</query>erg]]></query>",
      "<query>kjdfh<![CDATA[jh</query>erg]]></query>unsinn"
    };
    for (int i=0;i<tests.length;i++)
    {
      System.out.println(tests[i]);
      Reader testreader=new StringReader(tests[i]);
      Reader r=new QueryFilterReader(testreader);
      while (true)
      {
        try
        {
          int c=r.read();
          if (-1==c)
            break;
          System.out.print((char)c);
        }
        catch (IOException ex)
        {
          System.err.println(ex);
        }
      }
      System.out.println();
    }
  }
}