/*
Copyright (C) 1999/2000 Hendrik Lipka.
All rights reserved.

This program is free software; you can redistribute it and/or modify it under the terms 
of the GNU General Public License as published by the Free Software Foundation; 
either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY 
WARRANTY; without even the implied warranty of MERCHANTABILITY or 
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License 
for more details. 

You should have received a copy of the GNU General Public License along with this 
program; if not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
Cambridge, MA 02139, USA or see http://www.gnu.org

If you encounter problems or have a suggestion, please mail to:
hendrik.lipka@gmx.de
*/

package de.hendriklipka.ldse.server.query;

import org.w3c.dom.*;

import de.hendriklipka.ldse.common.Log;

public class QueryLimits
{
  public int _maxHits=0;
  public double _minLevel=0;
  public boolean _sorted=true;
  
  public QueryLimits(Node node)
  {
    try
    {
      NamedNodeMap nnl=node.getAttributes();
      Node attr=nnl.getNamedItem("minlevel");
      if (null!=attr)
      {
        String level=attr.getNodeValue();
        Log.log("queryserver","query",4,"Limits: minLevel="+level);
        _minLevel=Double.valueOf(level).doubleValue();
      }
      attr=nnl.getNamedItem("maxcount");
      if (null!=attr)
      {
        String level=attr.getNodeValue();
        Log.log("queryserver","query",4,"Limits: maxHits="+level);
        _maxHits=Integer.parseInt(level);
      }
      attr=nnl.getNamedItem("sorted");
      if (null!=attr)
      {
        String level=attr.getNodeValue().toLowerCase();
        Log.log("queryserver","query",4,"Limits: sorted="+level);
        if (level.equals("yes")||level.equals("1")||level.equals("true"))
        {
          _sorted=true;
        }
        else
        {
          _sorted=false;
        }
      }
      Log.log("queryserver","query",3,"Limits: maxHits="+_maxHits+" minLevel="+_minLevel);
    }
    catch (Exception ex)
    {
      ex.printStackTrace(Log.getWriter("queryserver","error",0));
    }
  }
  
  protected QueryLimits()
  {
  }
  
  public Object clone()
  {
    QueryLimits ql=new QueryLimits();
    ql._maxHits=_maxHits;
    ql._minLevel=_minLevel;
    return ql;
  }
}