/*
Copyright (C) 1999/2000 Hendrik Lipka.
All rights reserved.

This program is free software; you can redistribute it and/or modify it under the terms 
of the GNU General Public License as published by the Free Software Foundation; 
either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY 
WARRANTY; without even the implied warranty of MERCHANTABILITY or 
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License 
for more details. 

You should have received a copy of the GNU General Public License along with this 
program; if not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
Cambridge, MA 02139, USA or see http://www.gnu.org

If you encounter problems or have a suggestion, please mail to:
hendrik.lipka@gmx.de
*/

package de.hendriklipka.ldse.server.query;

import java.net.*;
import java.io.*;

import org.w3c.dom.*;

import HTTPClient.HTTPConnection;

import de.hendriklipka.ldse.common.Log;
import de.hendriklipka.ldse.server.dataserver.DBConnection;
import de.hendriklipka.ldse.server.broker.BrokerServer;

/**
  Behandelt ankommende Anfragen
*/
public class QueryServer extends Thread implements de.hendriklipka.ldse.common.Updateable, de.hendriklipka.ldse.common.StatisticWriter
{
  protected DBConnection _con=null;
  protected String _server=null;
  protected int _port=7622;
  protected boolean _reconfig;
  protected BrokerServer _brokerServer=null;
  
  protected static long _queries=0;
  static long _resultDocs=0;
  static long _queryWords=0;
  static long _queryTriplets=0;
  static long _time=0;
  static long _dbTime=0;
  
  public QueryServer(DBConnection con,Node configNode, BrokerServer brokerServer)
  {
    _con=con;
    _brokerServer=brokerServer;
    NodeList nl=configNode.getChildNodes();
    for (int i=0;i<nl.getLength();i++)
    {
      Node node=nl.item(i);
      if (node.getNodeType()!=node.ELEMENT_NODE)
        continue;
      String nn=node.getNodeName();
      if (nn.equals("port"))
      {
        _port=Integer.parseInt(de.hendriklipka.ldse.common.Util.getTextData(node));
      }
      else if (nn.equals("address"))
      {
        _server=de.hendriklipka.ldse.common.Util.getTextData(node);
      }
      else if (nn.equals("proxy"))
      {
        NamedNodeMap nnl=node.getAttributes();
        Node attr=nnl.getNamedItem("useproxy");
        String s=attr.getNodeValue();
        if (s.equals("no"))
          continue;
        attr=nnl.getNamedItem("proxyhost");
        String ph=attr.getNodeValue();
        attr=nnl.getNamedItem("proxyport");
        s=attr.getNodeValue();
        int pp=Integer.parseInt(s);
        HTTPConnection.setProxyServer(ph,pp);
        attr=nnl.getNamedItem("dontproxy");
        s=attr.getNodeValue();
        attr=nnl.getNamedItem("protocol");
        s=attr.getNodeValue();
      }
    }
  }
  
  public int getPort()
  {
    return _port;
  }
  
  public void run()
  {
    try
    {
      ServerSocket socket=null;
      if (null==_server)
      {
        socket=new ServerSocket(_port);
      }
      else
      {
        socket=new ServerSocket(_port,100,InetAddress.getByName(_server));
      }
      socket.setSoTimeout(1000);
      Log.log("queryserver","config",1,"awaiting client connections");
      while (true)
      {
        try
        {
          // auf Anfragen warten
          Socket s=socket.accept();
          // Handler erzeugen und starten
          ClientConnection cc=new ClientConnection(s,_con,_brokerServer);
          Log.log("queryserver","net",1,"thread for "+s.getInetAddress().getHostAddress()+":"+s.getPort()+" local: "+s.getLocalPort());
          cc.start();
          _queries++;
          cc=null;
        }
        catch (IOException ex) {}
        if (_reconfig)
        {
          Log.log("queryserver","config",1,"doing reconfiguration");
          _reconfig=false;
          ServerSocket oldServer=socket;
          try
          {
            if (null==_server)
            {
              socket=new ServerSocket(_port);
            }
            else
            {
              socket=new ServerSocket(_port,100,InetAddress.getByName(_server));
            }
            socket.setSoTimeout(1000);
          }
          catch (Exception ex)
          {
            socket=oldServer;
          }
          if (socket!=oldServer)
          {
            oldServer.close();
          }
        }
      }
    }
    catch (IOException ex) 
    {
      ex.printStackTrace(Log.getWriter("queryserver","error",0));
    }
  }
  
  public void update(Node configNode)
  {
    boolean newData=false;
    int port=_port;
    String server=_server;
    Log.log("queryserver","config",1,"reading reconfiguration");
    NodeList nl=configNode.getChildNodes();
    for (int i=0;i<nl.getLength();i++)
    {
      Node node=nl.item(i);
      if (node.getNodeType()!=node.ELEMENT_NODE)
        continue;
      String nn=node.getNodeName();
      if (nn.equals("port"))
      {
        _port=Integer.parseInt(de.hendriklipka.ldse.common.Util.getTextData(node));
        if (_port!=port)
          newData=true;
      }
      else if (nn.equals("address"))
      {
        _server=de.hendriklipka.ldse.common.Util.getTextData(node);
        if (_server!=server)
          newData=true;
      }
    }
    _reconfig=newData;
    if (!newData)
      Log.log("queryserver","config",1,"no new config data found!");
  }
  
  public void writeStatistic(PrintWriter pw) throws IOException
  {
    pw.println("*** QueryServer");
    pw.println("handled queries: "+_queries);
    pw.println("query words:     "+_queryWords);
    pw.println("query triplets:  "+_queryTriplets);
    pw.println("time:            "+_time+" ms");
    pw.println("resulting docs:  "+_resultDocs);
    pw.println();
  }
}