/*
Copyright (C) 1999/2000 Hendrik Lipka.
All rights reserved.

This program is free software; you can redistribute it and/or modify it under the terms 
of the GNU General Public License as published by the Free Software Foundation; 
either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY 
WARRANTY; without even the implied warranty of MERCHANTABILITY or 
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License 
for more details. 

You should have received a copy of the GNU General Public License along with this 
program; if not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
Cambridge, MA 02139, USA or see http://www.gnu.org

If you encounter problems or have a suggestion, please mail to:
hendrik.lipka@gmx.de
*/

package de.hendriklipka.ldse.server.query;

import java.util.*;

import com.objectspace.jgl.Container;
import com.objectspace.jgl.SList;
import com.objectspace.jgl.ForwardIterator;
import com.objectspace.jgl.OrderedSet;
import com.objectspace.jgl.BinaryPredicate;

import org.w3c.dom.*;

import de.hendriklipka.ldse.common.Log;
import de.hendriklipka.ldse.server.dataserver.DBConnection;
import de.hendriklipka.ldse.server.data.document.*;
import de.hendriklipka.ldse.common.ResultDescriptor;
import de.hendriklipka.ldse.server.broker.BrokerQuery;

// Behandelt einfache Anfragen
public class SimpleQueryProcessor extends de.hendriklipka.ldse.server.query.QueryProcessor
{
  protected String[] _words=null;
  protected TripletList _triplets=null;
  protected TripletList[] _tripletWords=null;
  protected DocumentDescriptor[] _tempResult=null;
  protected int _divisor=0;
  
	public SimpleQueryProcessor(Node query)
	{
		super(query);
	}

  // Woerter in Triplets zerlegen und als Liste speichern
  public void preprocessQuery()
  {
    _tripletWords=new TripletList[_words.length];
    TripletList list=new TripletList();
    for (int i=0;i<_words.length;i++)
    {
      TripletList tl=new TripletList(_words[i]);
      _tripletWords[i]=tl;
      list.addList(tl);
      QueryServer._queryWords++;
    }
    _triplets=list;
    Enumeration e=_triplets.getTriplets();
    while (e.hasMoreElements())
    {
      Triplet tr=(Triplet)e.nextElement();
      _divisor+=tr.count*tr.count;
      QueryServer._queryTriplets++;
    }
  }

  // eigentliches Resultat erzeugen
  public void postprocessQuery(DBConnection con)
  {
    int count=0;
    Container result=null;

    if (null!=_limits)
    {
      if (!_limits._sorted&&(0==_limits._maxHits))
      {
        result=new SList();
      }
    }
    if (null==result)
    {
      result=new OrderedSet(ResultDescriptor.getPredicate(),true);
    }
    Log.log("queryserver","query",3,"result has "+_tempResult.length+" elements");
    for (int i=0;i<_tempResult.length;i++)
    {
      DocumentDescriptor dd=_tempResult[i];
      TripletList list=dd.getList();
      double level=0;
      for (int j=0;j<_tripletWords.length;j++)
      {
        TripletList tl=_tripletWords[j];
        int min=Integer.MAX_VALUE;
        double value=0;
        
        Enumeration e=tl.getTriplets();
        // fuer jedes Wort in der Anfrage wird das minimale Triplet-Vorkommen
        // bestimmt
        // dieses wird dann zur Berechnung verwendet
        while (e.hasMoreElements())
        {
          Triplet tr=(Triplet)e.nextElement();
          // schauen, ob dieses Dokument es hat
          Triplet tr2=list.getTriplet(tr);
          // ja 
          if (null!=tr2)
          {
            if (tr2.count<min)
            {
              min=tr.count;
            }
            value+=tr2.count;
          }
          else
          {
            min=0;
          }
        }
        level+=value*min;
      }
      // nur relevante Dokumente als Ergebnis liefern
      if (0==level)
        continue;
      level=level/_divisor;
      dd=con.getDocumentDescriptor(dd.getID());
      ResultDescriptor rd=dd.createResultDescriptor();
      if (null!=_limits)
      {
        if (_limits._minLevel<=level)
        {
          rd.setRelevance(level);
          result.add(rd);
          count++;
        }
      }
      else // kein Limit -> auf jeden Fall einfuegen
      {
        count++;
        rd.setRelevance(level);
        result.add(rd);
      }
    }
    if (null!=_limits)
    {
      if (!_limits._sorted&&(0!=_limits._maxHits))
      {
        _result=result;
      }
    }
    if (null==_result)
    {
      _result=new SList();
      ForwardIterator it=result.start();
      int i=0;
      while (!it.atEnd()&&(i<count))
      {
        ResultDescriptor rd=(ResultDescriptor)it.get();
        _result.add(rd);
        i++;
        it.nextElement();
      }
    }
    Log.log("queryserver","query",3,"final result has "+_result.size()+" elements");
  }

  public void init()
  {
    String words=new String();;
    NodeList subs=_query.getChildNodes();
    for (int j=0;j<subs.getLength();j++)
    {
      Node attr=subs.item(j);
      if (attr.getNodeType()!=attr.TEXT_NODE)
        continue;
      Text val=(Text)attr;
      String s=val.getData().trim();
      if (0!=s.length())
      {
        words+=" "+s;
      }
    }
    words=words.trim();
    Log.log("queryserver","query",2,"final words: "+words);
    Vector v=new Vector();
    StringTokenizer st=new StringTokenizer(words);
    while (st.hasMoreElements())
    {
      String s=(String)st.nextElement();
      Log.log("queryserver","query",2,"adding word: "+s);
      v.addElement(s);
    }
    _words=new String[v.size()];
    for (int i=0;i<v.size();i++)
    {
      _words[i]=(String)v.elementAt(i);
    }
  }

  public void doQuery(DBConnection con)
  {
    _tempResult=con.queryDocuments(_triplets,null);
  }
  
  public BrokerQuery createBrokerQuery()
  {
    String s="<simple>\n";
    for (int i=0;i<_words.length;i++)
    {
      s+=_words[i]+"\n";
    }
    s+="</simple>\n";

    return new BrokerQuery(s,this,_triplets,_limits);
  }
    
  public void close()
  {
  }
}